@@ -34,14 +34,14 @@ static GdkGC *gdk_pixmap_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValuesMask  mask);
 static void   gdk_pixmap_draw_rectangle (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 gint             x,
 					 gint             y,
 					 gint             width,
 					 gint             height);
 static void   gdk_pixmap_draw_arc       (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 gint             x,
 					 gint             y,
 					 gint             width,
@@ -50,7 +50,7 @@ static void   gdk_pixmap_draw_arc       (GdkDrawable     *drawable,
 					 gint             angle2);
 static void   gdk_pixmap_draw_polygon   (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 GdkPoint        *points,
 					 gint             npoints);
 static void   gdk_pixmap_draw_text      (GdkDrawable     *drawable,
@@ -200,7 +200,7 @@ gdk_pixmap_class_init (GdkPixmapObjectClass *klass)
   drawable_class->draw_lines = gdk_pixmap_draw_lines;
   drawable_class->draw_glyphs = gdk_pixmap_draw_glyphs;
   drawable_class->draw_image = gdk_pixmap_draw_image;
-  drawable_class->_draw_pixbuf = gdk_pixmap_draw_pixbuf;
+  drawable_class->draw_pixbuf = gdk_pixmap_draw_pixbuf;
   drawable_class->get_depth = gdk_pixmap_real_get_depth;
   drawable_class->get_screen = gdk_pixmap_real_get_screen;
   drawable_class->get_size = gdk_pixmap_real_get_size;
@@ -233,7 +233,7 @@ gdk_pixmap_create_gc (GdkDrawable     *drawable,
 static void
 gdk_pixmap_draw_rectangle (GdkDrawable *drawable,
 			   GdkGC       *gc,
-			   gint         filled,
+			   gboolean     filled,
 			   gint         x,
 			   gint         y,
 			   gint         width,
@@ -248,7 +248,7 @@ gdk_pixmap_draw_rectangle (GdkDrawable *drawable,
 static void
 gdk_pixmap_draw_arc (GdkDrawable *drawable,
 		     GdkGC       *gc,
-		     gint         filled,
+		     gboolean     filled,
 		     gint         x,
 		     gint         y,
 		     gint         width,
@@ -266,7 +266,7 @@ gdk_pixmap_draw_arc (GdkDrawable *drawable,
 static void
 gdk_pixmap_draw_polygon (GdkDrawable *drawable,
 			 GdkGC       *gc,
-			 gint         filled,
+			 gboolean     filled,
 			 GdkPoint    *points,
 			 gint         npoints)
 {
@@ -402,9 +402,9 @@ gdk_pixmap_draw_pixbuf (GdkDrawable     *drawable,
 {
   GdkPixmapObject *private = (GdkPixmapObject *)drawable;
 
-  _gdk_draw_pixbuf (private->impl, gc, pixbuf,
-		    src_x, src_y, dest_x, dest_y, width, height,
-		    dither, x_dither, y_dither);
+  gdk_draw_pixbuf (private->impl, gc, pixbuf,
+		   src_x, src_y, dest_x, dest_y, width, height,
+		   dither, x_dither, y_dither);
 }
 
 static void
@@ -470,10 +470,10 @@ gdk_pixmap_copy_to_image (GdkDrawable     *drawable,
 {
   g_return_val_if_fail (GDK_IS_PIXMAP (drawable), NULL);
   
-  return _gdk_drawable_copy_to_image (((GdkPixmapObject*)drawable)->impl,
-				      image,
-				      src_x, src_y, dest_x, dest_y,
-				      width, height);
+  return gdk_drawable_copy_to_image (((GdkPixmapObject*)drawable)->impl,
+				     image,
+				     src_x, src_y, dest_x, dest_y,
+				     width, height);
 }
 
 static GdkBitmap *
@@ -496,13 +496,15 @@ make_solid_mask (GdkScreen *screen, gint width, gint height)
   return bitmap;
 }
 
-#define PACKED_COLOR(c) ((((c)->red & 0xff) << 8) | ((c)->green & 0xff) | ((c)->blue >> 8))
+#define PACKED_COLOR(c) ((((c)->red   & 0xff00)  << 8) |   \
+			  ((c)->green & 0xff00)        |   \
+			  ((c)->blue             >> 8))
 
 static GdkPixmap *
-gdk_pixmap_colormap_new_from_pixbuf (GdkColormap *colormap,
-				     GdkBitmap  **mask,
-				     GdkColor    *transparent_color,
-				     GdkPixbuf   *pixbuf)
+gdk_pixmap_colormap_new_from_pixbuf (GdkColormap    *colormap,
+				     GdkBitmap     **mask,
+				     const GdkColor *transparent_color,
+				     GdkPixbuf      *pixbuf)
 {
   GdkPixmap *pixmap;
   GdkPixbuf *render_pixbuf;
@@ -527,12 +529,11 @@ gdk_pixmap_colormap_new_from_pixbuf (GdkColormap *colormap,
   else
     render_pixbuf = pixbuf;
 
-  tmp_gc = gdk_gc_new (pixmap);
-  gdk_pixbuf_render_to_drawable (render_pixbuf, pixmap, tmp_gc, 0, 0, 0, 0,
-				 gdk_pixbuf_get_width (render_pixbuf),
-				 gdk_pixbuf_get_height (render_pixbuf),
-				 GDK_RGB_DITHER_NORMAL, 0, 0);
-  g_object_unref (tmp_gc);
+  tmp_gc = _gdk_drawable_get_scratch_gc (pixmap, FALSE);
+  gdk_draw_pixbuf (pixmap, tmp_gc, render_pixbuf, 0, 0, 0, 0,
+		   gdk_pixbuf_get_width (render_pixbuf),
+		   gdk_pixbuf_get_height (render_pixbuf),
+		   GDK_RGB_DITHER_NORMAL, 0, 0);
 
   if (render_pixbuf != pixbuf)
     g_object_unref (render_pixbuf);
@@ -549,21 +550,21 @@ gdk_pixmap_colormap_new_from_pixbuf (GdkColormap *colormap,
 }
 
 GdkPixmap*
-gdk_pixmap_colormap_create_from_xpm (GdkWindow   *window,
-				     GdkColormap *colormap,
-				     GdkBitmap  **mask,
-				     GdkColor    *transparent_color,
-				     const gchar *filename)
+gdk_pixmap_colormap_create_from_xpm (GdkDrawable    *drawable,
+				     GdkColormap    *colormap,
+				     GdkBitmap     **mask,
+				     const GdkColor *transparent_color,
+				     const gchar    *filename)
 {
   GdkPixbuf *pixbuf;
   GdkPixmap *pixmap;
 
-  g_return_val_if_fail (window != NULL || colormap != NULL, NULL);
-  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
+  g_return_val_if_fail (drawable != NULL || colormap != NULL, NULL);
+  g_return_val_if_fail (drawable == NULL || GDK_IS_DRAWABLE (drawable), NULL);
   g_return_val_if_fail (colormap == NULL || GDK_IS_COLORMAP (colormap), NULL);
 
   if (colormap == NULL)
-    colormap = gdk_drawable_get_colormap (window);
+    colormap = gdk_drawable_get_colormap (drawable);
   
   pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
   if (!pixbuf)
@@ -577,31 +578,31 @@ gdk_pixmap_colormap_create_from_xpm (GdkWindow   *window,
 }
 
 GdkPixmap*
-gdk_pixmap_create_from_xpm (GdkWindow  *window,
-			    GdkBitmap **mask,
-			    GdkColor   *transparent_color,
-			    const gchar *filename)
+gdk_pixmap_create_from_xpm (GdkDrawable    *drawable,
+			    GdkBitmap     **mask,
+			    const GdkColor *transparent_color,
+			    const gchar    *filename)
 {
-  return gdk_pixmap_colormap_create_from_xpm (window, NULL, mask,
+  return gdk_pixmap_colormap_create_from_xpm (drawable, NULL, mask,
 					      transparent_color, filename);
 }
 
 GdkPixmap*
-gdk_pixmap_colormap_create_from_xpm_d (GdkWindow  *window,
-				       GdkColormap *colormap,
-				       GdkBitmap **mask,
-				       GdkColor   *transparent_color,
-				       gchar     **data)
+gdk_pixmap_colormap_create_from_xpm_d (GdkDrawable    *drawable,
+				       GdkColormap    *colormap,
+				       GdkBitmap     **mask,
+				       const GdkColor *transparent_color,
+				       gchar         **data)
 {
   GdkPixbuf *pixbuf;
   GdkPixmap *pixmap;
 
-  g_return_val_if_fail (window != NULL || colormap != NULL, NULL);
-  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
+  g_return_val_if_fail (drawable != NULL || colormap != NULL, NULL);
+  g_return_val_if_fail (drawable == NULL || GDK_IS_DRAWABLE (drawable), NULL);
   g_return_val_if_fail (colormap == NULL || GDK_IS_COLORMAP (colormap), NULL);
 
   if (colormap == NULL)
-    colormap = gdk_drawable_get_colormap (window);
+    colormap = gdk_drawable_get_colormap (drawable);
   
   pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **)data);
   if (!pixbuf)
@@ -615,12 +616,12 @@ gdk_pixmap_colormap_create_from_xpm_d (GdkWindow  *window,
 }
 
 GdkPixmap*
-gdk_pixmap_create_from_xpm_d (GdkWindow  *window,
-			      GdkBitmap **mask,
-			      GdkColor   *transparent_color,
-			      gchar     **data)
+gdk_pixmap_create_from_xpm_d (GdkDrawable    *drawable,
+			      GdkBitmap     **mask,
+			      const GdkColor *transparent_color,
+			      gchar         **data)
 {
-  return gdk_pixmap_colormap_create_from_xpm_d (window, NULL, mask,
+  return gdk_pixmap_colormap_create_from_xpm_d (drawable, NULL, mask,
 						transparent_color, data);
 }
 
