@@ -188,12 +188,12 @@ gdk_cursor_new_for_display (GdkDisplay    *display,
  * Return value: a new #GdkCursor.
  **/
 GdkCursor*
-gdk_cursor_new_from_pixmap (GdkPixmap *source,
-			    GdkPixmap *mask,
-			    GdkColor  *fg,
-			    GdkColor  *bg,
-			    gint       x,
-			    gint       y)
+gdk_cursor_new_from_pixmap (GdkPixmap      *source,
+			    GdkPixmap      *mask,
+			    const GdkColor *fg,
+			    const GdkColor *bg,
+			    gint            x,
+			    gint            y)
 {
   GdkCursorPrivate *private;
   GdkCursor *cursor;
@@ -560,7 +560,7 @@ gdk_display_supports_cursor_color (GdkDisplay    *display)
 guint     
 gdk_display_get_default_cursor_size (GdkDisplay    *display)
 {
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), 0);
   
   /* no idea, really */
   return 20; 
