@@ -55,11 +55,6 @@ typedef struct _GdkIOClosure GdkIOClosure;
 typedef struct _GdkDisplaySource GdkDisplaySource;
 typedef struct _GdkEventTypeX11 GdkEventTypeX11;
 
-#define DOUBLE_CLICK_TIME      250
-#define TRIPLE_CLICK_TIME      500
-#define DOUBLE_CLICK_DIST      5
-#define TRIPLE_CLICK_DIST      5
-
 struct _GdkIOClosure
 {
   GdkInputFunction function;
@@ -150,6 +145,22 @@ gdk_check_xpending (GdkDisplay *display)
  * Functions for maintaining the event queue *
  *********************************************/
 
+static void
+refcounted_grab_server (Display *xdisplay)
+{
+  GdkDisplay *display = gdk_x11_lookup_xdisplay (xdisplay);
+
+  gdk_x11_display_grab (display);
+}
+
+static void
+refcounted_ungrab_server (Display *xdisplay)
+{
+  GdkDisplay *display = gdk_x11_lookup_xdisplay (xdisplay);
+  
+  gdk_x11_display_ungrab (display);
+}
+
 void
 _gdk_x11_events_init_screen (GdkScreen *screen)
 {
@@ -163,6 +174,10 @@ _gdk_x11_events_init_screen (GdkScreen *screen)
 						       gdk_xsettings_notify_cb,
 						       gdk_xsettings_watch_cb,
 						       screen);
+  xsettings_client_set_grab_func (screen_x11->xsettings_client,
+				  refcounted_grab_server);
+  xsettings_client_set_ungrab_func (screen_x11->xsettings_client,
+				    refcounted_ungrab_server);
   screen_x11->xsettings_in_init = FALSE;
 }
 
@@ -651,10 +666,31 @@ translate_key_event (GdkDisplay *display,
   return;
 }
 
+/**
+ * gdk_x11_register_standard_event_type:
+ * @display: a #GdkDisplay
+ * @event_base: first event type code to register
+ * @n_events: number of event type codes to register
+ * 
+ * Registers interest in receiving extension events with type codes
+ * between @event_base and <literal>event_base + n_events - 1</literal>.
+ * The registered events must have the window field in the same place
+ * as core X events (this is not the case for e.g. XKB extension events).
+ *
+ * If an event type is registered, events of this type will go through
+ * global and window-specific filters (see gdk_window_add_filter()). 
+ * Unregistered events will only go through global filters.
+ * GDK may register the events of some X extensions on its own.
+ * 
+ * This function should only be needed in unusual circumstances, e.g.
+ * when filtering XInput extension events on the root window.
+ *
+ * Since: 2.4
+ **/
 void
-_gdk_x11_register_event_type (GdkDisplay          *display,
-			      gint                 event_base,
-			      gint                 n_events)
+gdk_x11_register_standard_event_type (GdkDisplay          *display,
+				      gint                 event_base,
+				      gint                 n_events)
 {
   GdkEventTypeX11 *event_type;
   GdkDisplayX11 *display_x11;
@@ -1969,11 +2005,12 @@ gdk_wm_protocols_filter (GdkXEvent *xev,
   else if ((Atom) xevent->xclient.data.l[0] == gdk_x11_get_xatom_by_name_for_display (display, "WM_TAKE_FOCUS"))
     {
       GdkToplevelX11 *toplevel = _gdk_x11_window_get_toplevel (event->any.window);
+      GdkWindowObject *private = (GdkWindowObject *)win;
 
       /* There is no way of knowing reliably whether we are viewable;
        * _gdk_x11_set_input_focus_safe() traps errors asynchronously.
        */
-      if (toplevel)
+      if (toplevel && private->accept_focus)
 	_gdk_x11_set_input_focus_safe (display, toplevel->focus_window,
 				       RevertToParent,
 				       xevent->xclient.data.l[1]);
@@ -2369,15 +2406,19 @@ fetch_net_wm_check_window (GdkScreen *screen)
   
   /* Find out if this WM goes away, so we can reset everything. */
   XSelectInput (screen_x11->xdisplay, *xwindow, StructureNotifyMask);
+  gdk_display_sync (display);
 
-  screen_x11->wmspec_check_window = *xwindow;
-  XFree (xwindow);
-
-  screen_x11->need_refetch_net_supported = TRUE;
-  screen_x11->need_refetch_wm_name = TRUE;
-  
-  /* Careful, reentrancy */
-  _gdk_x11_screen_window_manager_changed (GDK_SCREEN (screen_x11));
+  if (gdk_error_trap_pop () == Success)
+    {
+      screen_x11->wmspec_check_window = *xwindow;
+      XFree (xwindow);
+      
+      screen_x11->need_refetch_net_supported = TRUE;
+      screen_x11->need_refetch_wm_name = TRUE;
+      
+      /* Careful, reentrancy */
+      _gdk_x11_screen_window_manager_changed (GDK_SCREEN (screen_x11));
+    }
 }
 
 /**
@@ -2418,6 +2459,8 @@ gdk_x11_screen_get_window_manager_name (GdkScreen *screen)
           guchar *name;
           
           name = NULL;
+
+	  gdk_error_trap_push ();
           
           XGetWindowProperty (GDK_DISPLAY_XDISPLAY (screen_x11->display),
                               screen_x11->wmspec_check_window,
@@ -2569,6 +2612,7 @@ static struct
   const char *gdk_name;
 } settings_map[] = {
   { "Net/DoubleClickTime", "gtk-double-click-time" },
+  { "Net/DoubleClickDistance", "gtk-double-click-distance" },
   { "Net/DndDragThreshold", "gtk-dnd-drag-threshold" },
   { "Gtk/CanChangeAccels", "gtk-can-change-accels" },
   { "Gtk/ColorPalette", "gtk-color-palette" },
@@ -2583,6 +2627,13 @@ static struct
   { "Net/CursorBlinkTime", "gtk-cursor-blink-time" },
   { "Net/ThemeName", "gtk-theme-name" },
   { "Net/IconThemeName", "gtk-icon-theme-name" },
+  { "Gtk/ButtonImages", "gtk-button-images" },
+  { "Gtk/MenuImages", "gtk-menu-images" },
+  { "Xft/Antialias", "gtk-xft-antialias" },
+  { "Xft/Hinting", "gtk-xft-hinting" },
+  { "Xft/HintStyle", "gtk-xft-hintstyle" },
+  { "Xft/RGBA", "gtk-xft-rgba" },
+  { "Xft/DPI", "gtk-xft-dpi" },
 };
 
 static void
@@ -2610,7 +2661,6 @@ gdk_xsettings_notify_cb (const char       *name,
 	new_event.setting.name = (char *)settings_map[i].gdk_name;
 	break;
       }
-
   
   if (!new_event.setting.name)
     return;
