@@ -359,7 +359,7 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
       gtk_paint_option (widget->style,
                         window,
                         state, shadow,
-                        cell_area, widget, "cellradio",
+                        expose_area, widget, "cellradio",
                         cell_area->x + x_offset + cell->xpad,
                         cell_area->y + y_offset + cell->ypad,
                         width - 1, height - 1);
@@ -369,7 +369,7 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
       gtk_paint_check (widget->style,
                        window,
                        state, shadow,
-                       cell_area, widget, "cellcheck",
+                       expose_area, widget, "cellcheck",
                        cell_area->x + x_offset + cell->xpad,
                        cell_area->y + y_offset + cell->ypad,
                        width - 1, height - 1);
