@@ -223,11 +223,11 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
       pos = gtk_editable_get_position (editable);
       prefix = gtk_editable_get_chars (editable, 0, pos);
 
-      g_completion_complete (cmpl, prefix, &nprefix);
+      g_completion_complete_utf8 (cmpl, prefix, &nprefix);
 
       if (nprefix && strlen (nprefix) > strlen (prefix)) 
 	{
-	  gtk_editable_insert_text (editable, nprefix + pos, 
+	  gtk_editable_insert_text (editable, g_utf8_offset_to_pointer (nprefix, pos), 
 				    strlen (nprefix) - strlen (prefix), &pos);
 	  gtk_editable_set_position (editable, pos);
 	}
@@ -1109,13 +1109,13 @@ gtk_combo_set_use_arrows_always (GtkCombo * combo, gboolean val)
 }
 
 void
-gtk_combo_set_popdown_strings (GtkCombo * combo, GList * strings)
+gtk_combo_set_popdown_strings (GtkCombo *combo, 
+			       GList    *strings)
 {
   GList *list;
   GtkWidget *li;
 
   g_return_if_fail (GTK_IS_COMBO (combo));
-  g_return_if_fail (strings != NULL);
 
   gtk_combo_popdown_list (combo);
 
