@@ -83,11 +83,11 @@ typedef enum
 
 
 #define GTK_TYPE_DIALOG                  (gtk_dialog_get_type ())
-#define GTK_DIALOG(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_DIALOG, GtkDialog))
-#define GTK_DIALOG_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_DIALOG, GtkDialogClass))
-#define GTK_IS_DIALOG(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_DIALOG))
-#define GTK_IS_DIALOG_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DIALOG))
-#define GTK_DIALOG_GET_CLASS(obj)        (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_DIALOG, GtkDialogClass))
+#define GTK_DIALOG(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_DIALOG, GtkDialog))
+#define GTK_DIALOG_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_DIALOG, GtkDialogClass))
+#define GTK_IS_DIALOG(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_DIALOG))
+#define GTK_IS_DIALOG_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DIALOG))
+#define GTK_DIALOG_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_DIALOG, GtkDialogClass))
 
 
 typedef struct _GtkDialog        GtkDialog;
@@ -97,6 +97,7 @@ struct _GtkDialog
 {
   GtkWindow window;
 
+  /*< public >*/
   GtkWidget *vbox;
   GtkWidget *action_area;
 
@@ -122,7 +123,7 @@ struct _GtkDialogClass
 };
 
 
-GtkType    gtk_dialog_get_type (void) G_GNUC_CONST;
+GType      gtk_dialog_get_type (void) G_GNUC_CONST;
 GtkWidget* gtk_dialog_new      (void);
 
 GtkWidget* gtk_dialog_new_with_buttons (const gchar     *title,
@@ -158,6 +159,11 @@ void gtk_dialog_response           (GtkDialog *dialog,
 /* Returns response_id */
 gint gtk_dialog_run                (GtkDialog *dialog);
 
+
+/* For private use only */
+void _gtk_dialog_set_ignore_separator (GtkDialog *dialog,
+				       gboolean   ignore_separator);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
